// ================================================
// Fader
// ================================================
function Fader(nodeID, flag, frontID) {
	LOG__("Object: Fader");
	this.nodeID      = nodeID;
	this.target      = document.getElementById(nodeID);
	this.frontID     = frontID;
	this.frontTarget = (frontID)? document.getElementById(frontID) : null;
	this.inDuration  = 500;
	this.outDuration = 500;
	this.duration    = 500;
	this.interval    = 50; // 13
	this.delay       = 0;
	this.starttime   = 0;
	this.from        = 0.0;
	this.to          = 1.0;
	this.now         = flag ? 1.0 : 0.0;
	this.flag        = flag;
	this.timer       = null;
	this.target.style.opacity = this.now;

	// Fade in/out events
	if (!this.frontTarget) return;
	var localThis = this;
	this.mouseOverHandler = function(event) { localThis.mouseOver(event); };
	this.mouseOutHandler  = function(event) { localThis.mouseOut(event); };

	this.frontTarget.addEventListener("mouseover", this.mouseOverHandler, false);
	this.frontTarget.addEventListener("mouseout",  this.mouseOutHandler,  false);
}

Fader.prototype.mouseOver = function(event) {
	if (this.flag) return;
	LOG2__("Fader.mouseOver");

	this.fadeIn();
	event.stopPropagation();
	event.preventDefault();
}

Fader.prototype.mouseOut = function(event) {
	if (!this.flag) return;
	LOG2__("Fader.mouseOut");

	this.fadeOut();
	event.stopPropagation();
	event.preventDefault();
}

Fader.prototype.fadeIn = function() {
	LOG3__("Fader.fadeIn");
	var localThis = this;
	if (!this.flag) {
		if (this.timer != null) {
			clearInterval(this.timer);
			this.timer = null;
		}
		var starttime = (new Date).getTime() - this.interval;
		this.duration = this.inDuration;
		this.starttime = starttime;
		this.from = this.now;
		this.to = 1.0;
		this.flag = true;
		this.timer = setTimeout(function() { localThis.animate() }, this.delay);
    }
}

Fader.prototype.fadeOut = function() {
	LOG3__("Fader.fadeOut");
	var localThis = this;
	if (this.flag) {
		if (this.timer != null) {
			clearInterval(this.timer);
			this.timer = null;
		}
		var starttime = (new Date).getTime() - this.interval;
		this.duration = this.outDuration;
		this.starttime = starttime;
		this.from = this.now;
		this.to = 0.0;
		this.flag = false;
		this.timer = setTimeout(function() { localThis.animate() }, this.delay);
	}
}

Fader.prototype.setOpacity = function(opc) {
	LOG3__("Fader.setOpacity");
	if (this.timer != null) {
		clearInterval(this.timer);
		this.timer = null;
	}
	if ((0 < opc) && (opc <= 1.0)) {
		this.now = opc;
		this.flag = true;
	}
	if (opc == 0) {
		this.now = opc;
		this.flag = false;
	}
	this.target.style.opacity = this.now;
}

Fader.prototype.animate = function() {
	var time = (new Date).getTime();
	var T = limit_3(time - this.starttime, 0, this.duration + this.delay);
	var ease;
	var localThis = this;

	if (T >= (this.duration + this.delay)) {
		clearTimeout(this.timer);
		this.timer = null;
		this.now = this.to;
	} else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / (this.duration + this.delay)));
		this.now = computeNextFloat(this.from, this.to, ease);
		this.timer = setTimeout(function() { localThis.animate() }, this.interval);
	}
	this.target.style.opacity = this.now;
}

function limit_3(a, b, c) {
	return a < b ? b : (a > c ? c : a);
}

function computeNextFloat(from, to, ease) {
	return from + (to - from) * ease;
}
