// ================================================
// Buttons
// ================================================
function Button(nodeID, imageDefault, imageClicked, imageAlt) {
	LOG__("Object: Button " + nodeID);
	this.nodeID      = nodeID;
	this.target      = document.getElementById(nodeID);

	this.imageDefault = imageDefault;
	this.imageClicked = imageClicked;
	this.imageAlt     = imageAlt;

	this.pressed   = false;
	this.target.setAttribute("src", this.imageDefault);
	this.target.style.appleDashboardRegion = "dashboard-region(control rectangle 0 0 0 0)";
	
	var localThis = this;
	this.mouseDownHandler = function(event) { localThis.buttonDown(event); };
	this.mouseUpHandler   = function(event) { localThis.buttonUp(event);   };
	this.mouseOutHandler  = function(event) { localThis.buttonOut(event);  };

	this.target.addEventListener("mousedown", this.mouseDownHandler, false);
}

Button.prototype.buttonDown = function(event) {
	LOG2__("Button.buttonDown");
	this.setPressed(true);
	this.target.addEventListener("mouseup",   this.mouseUpHandler,   false);
	this.target.addEventListener("mouseout",  this.mouseOutHandler,  false);

	event.stopPropagation();
	event.preventDefault();
}

Button.prototype.buttonUp = function(event) {
	LOG2__("Button.buttonUp");
	if (this.setPressed)
		this.onClick();
	this.setPressed(false);
	this.target.removeEventListener("mouseup",   this.mouseUpHandler,   false);
	this.target.removeEventListener("mouseout",  this.mouseOutHandler,  false);

	event.stopPropagation();
	event.preventDefault();
}

Button.prototype.buttonOut = function(event) {
	LOG2__("Button.buttonOut");
	this.setPressed(false);

	this.target.removeEventListener("mouseup",   this.mouseUpHandler,   false);
	this.target.removeEventListener("mouseout",  this.mouseOutHandler,  false);
	event.stopPropagation();
	event.preventDefault();
}

Button.prototype.setPressed = function(flag) {
	if (flag == this.pressed)
		return;

	if (flag) {
		if (this.imageClicked)
			this.target.setAttribute("src", this.imageClicked);
	} else {
		if (this.imageDefault)
			this.target.setAttribute("src", this.imageDefault);
	}
	this.pressed = flag;
}

Button.prototype.onClick = function(flag) {
	LOG2__("Button.onClick");
}

Button.prototype.keyDown = function(flag) {
	LOG2__("Button.keyDown");

	this.setPressed(true);
}

Button.prototype.keyUp = function(flag) {
	LOG2__("Button.keyUp");

	if (this.setPressed)
		this.onClick();
	this.setPressed(false);
}
